package net.sf.sido.spring;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.sido.DataFactory;
import net.sf.sido.DataSchema;
import net.sf.sido.factory.DataFactoryReader;
import net.sf.sido.factory.json.JSONDataFactoryReader;
import net.sf.sido.factory.sido.SiDOLDataFactoryReader;
import net.sf.sido.support.DataFactories;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.Resource;

public class DataFactoryLoader {

	private final Logger logger = LoggerFactory
			.getLogger(DataFactoryLoader.class);

	private final ApplicationContext context;

	private Map<String, DataFactoryReader> readers = new HashMap<String, DataFactoryReader>();

	private String paths;
	private Comparator<Resource> comparator;
	private DataFactory dataFactory;

	@Autowired
	public DataFactoryLoader(ApplicationContext context) {
		this.context = context;
		// Default readers
		readers.put("json", new JSONDataFactoryReader());
		readers.put("sidol", new SiDOLDataFactoryReader());
	}

	public Comparator<Resource> getComparator() {
		return comparator;
	}

	public DataFactory getDataFactory() {
		return dataFactory;
	}

	public String getPaths() {
		return paths;
	}

	public Map<String, DataFactoryReader> getReaders() {
		return readers;
	}

	public DataFactory load() throws IOException {
		// Creates the factory if needed
		DataFactory factory;
		if (dataFactory != null) {
			factory = dataFactory;
		} else {
			factory = DataFactories.typedFactory();
		}
		// Gets the list of all resources
		List<Resource> resources = new ArrayList<Resource>();
		// Gets the list of paths
		logger.debug("Loading from paths {}", paths);
		String[] pathArray = StringUtils.split(paths, ",; \t\r\n");
		for (String path : pathArray) {
			logger.debug("Getting resources from path {}", path);
			// Loads the resources
			Resource[] resourcesArray = context.getResources(path);
			resources.addAll(Arrays.asList(resourcesArray));
		}
		// Sorts the resources
		if (comparator != null) {
            logger.debug("Sorting all resources using {}", comparator);
			Collections.sort(resources, comparator);
		}
        // Logs the full list of paths
        if (logger.isDebugEnabled()) {
            logger.debug("All resources to load...");
            int index = 1;
            for (Resource resource : resources) {
                logger.debug("\t{} - {}", index++, resource);
            }
        }
		// For each resource
		for (Resource resource : resources) {
			// Log
			logger.info("Loading schema from {}", resource.getDescription());
			// Resource name
			String name = resource.getFilename();
			// Gets the extension
			String extension = StringUtils.substringAfterLast(name, ".");
			if (StringUtils.isBlank(extension)) {
				throw new IOException(
						String.format(
								"Cannot load %s because no file extension is available to determine the loader type.",
								resource.getDescription()));
			}
			extension = StringUtils.lowerCase(extension);
			logger.debug("Resource extension is [{}]", extension);
			// Gets the associated loader
			DataFactoryReader reader = readers.get(extension);
			if (reader == null) {
				throw new IOException(
						String.format("Cannot find any DataFactoryReader for extension %s. Please register a reader for this extension."));
			}
			logger.debug("Using reader {}", reader);
			// Reads the schema
			URL url = resource.getURL();
			logger.debug("Reading the schema from URL {}", url);
			DataSchema schema = reader.read(url, factory);
			// Log
			if (logger.isInfoEnabled()) {
				logger.info(
						"Schema [{}] has been read from {} for following types: {}",
						new Object[] { schema.getName(),
								resource.getDescription(), schema.getTypes() });
			}
		}
		// OK
		return factory;
	}

	public void setComparator(Comparator<Resource> comparator) {
		this.comparator = comparator;
	}

	public void setDataFactory(DataFactory dataFactory) {
		this.dataFactory = dataFactory;
	}

	@Required
	public void setPaths(String paths) {
		this.paths = paths;
	}

	public void setReaders(Map<String, DataFactoryReader> readers) {
		this.readers = readers;
	}

}
